// BenLincoln.DaVinci.GUI - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.GUI.

// BenLincoln.DaVinci.GUI is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci.GUI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci.GUI (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace BenLincoln.DaVinci.GUI
{
    public partial class EngineStatusDisplay : UserControl, IDisposable
    {
        public enum EngineDisplayStatus
        {
            Normal,
            Paused,
            Finished,
            Failed
        }

        // milliseconds between iterating through the text available for the status display labels
        protected int _StatusTimerInterval = 5000;

        protected string _DirectoryName;
        protected int _TotalImages;
        protected int _ProcessedImages;
        protected string _CurrentStatus;
        protected string _EstimatedTimeRemainingString;
        protected float _EstimatedTimeRemainingConfidence;
        protected int _ProcessCount;
        protected string _CurrentMemoryUsageString;
        protected string _MaxMemoryUsageString;
        protected string _AverageMemoryUsageString;
        protected string _MedianMemoryUsageString;
        protected string _CurrentProcessorUsageString;
        protected string _MaxProcessorUsageString;
        protected string _AverageProcessorUsageString;
        protected string _MedianProcessorUsageString;

        protected bool _DisplayPerformanceData;
        protected bool _DisplayThumbnails;

        protected bool _HasRegisteredError;
        protected EngineDisplayStatus _DisplayStatus;

        protected int _Status1Index;
        protected int _Status2Index;
        protected Timer _StatusTimer;

        protected bool _Disposed;

        #region Properties

        public string DirectoryName
        {
            get
            {
                return _DirectoryName;
            }
            set
            {
                _DirectoryName = value;
                lblDirectoryName.Text = "Directory: " + _DirectoryName;
                this.Invalidate();
            }
        }

        public int TotalImages
        {
            get
            {
                return _TotalImages;
            }
            set
            {
                _TotalImages = value;
                if (_TotalImages > 0)
                {
                    pbProgress.Maximum = _TotalImages;
                }
                else
                {
                    pbProgress.Maximum = 1;
                }
                UpdateImagesProcessedLabel();
                this.Invalidate();
            }
        }

        public int ProcessedImages
        {
            get
            {
                return _ProcessedImages;
            }
            set
            {
                _ProcessedImages = value;
                if ((pbProgress.Enabled) && (_ProcessedImages > 0))
                {
                    if (_ProcessedImages < pbProgress.Maximum)
                    {
                        pbProgress.Value = _ProcessedImages;
                    }
                    else
                    {
                        pbProgress.Value = pbProgress.Maximum;
                    }
                }
                else
                {
                    pbProgress.Value = 0;
                }
                UpdateImagesProcessedLabel();
                this.Invalidate();
            }
        }

        public string CurrentStatus
        {
            get
            {
                return _CurrentStatus;
            }
            set
            {
                _CurrentStatus = value;
            }
        }

        public string EstimatedTimeRemainingString
        {
            get
            {
                return _EstimatedTimeRemainingString;
            }
            set
            {
                _EstimatedTimeRemainingString = value;
                //UpdateTimeRemainingLabel();
                //this.Invalidate();
            }
        }

        public float EstimatedTimeRemainingConfidence
        {
            get
            {
                return _EstimatedTimeRemainingConfidence;
            }
            set
            {
                _EstimatedTimeRemainingConfidence = value;
                //UpdateTimeRemainingLabel();
                //this.Invalidate();
            }
        }

        public int ProcessCount
        {
            get
            {
                return _ProcessCount;
            }
            set
            {
                _ProcessCount = value;
                //lblProcessCount.Text = "Process Count: " + _ProcessCount.ToString();
                //this.Invalidate();
            }
        }

        public string CurrentMemoryUsageString
        {
            get
            {
                return _CurrentMemoryUsageString;
            }
            set
            {
                _CurrentMemoryUsageString = value;
                //UpdateMemoryStatsLabel();
            }
        }

        public string MaxMemoryUsageString
        {
            get
            {
                return _MaxMemoryUsageString;
            }
            set
            {
                _MaxMemoryUsageString = value;
                //UpdateMemoryStatsLabel();
            }
        }

        public string AverageMemoryUsageString
        {
            get
            {
                return _AverageMemoryUsageString;
            }
            set
            {
                _AverageMemoryUsageString = value;
                //UpdateMemoryStatsLabel();
            }
        }

        public string MedianMemoryUsageString
        {
            get
            {
                return _MedianMemoryUsageString;
            }
            set
            {
                _MedianMemoryUsageString = value;
                //UpdateMemoryStatsLabel();
            }
        }

        public string CurrentProcessorUsageString
        {
            get
            {
                return _CurrentProcessorUsageString;
            }
            set
            {
                _CurrentProcessorUsageString = value;
                //UpdateProcessorStatsLabel();
            }
        }

        public string MaxProcessorUsageString
        {
            get
            {
                return _MaxProcessorUsageString;
            }
            set
            {
                _MaxProcessorUsageString = value;
                //UpdateProcessorStatsLabel();
            }
        }

        public string AverageProcessorUsageString
        {
            get
            {
                return _AverageProcessorUsageString;
            }
            set
            {
                _AverageProcessorUsageString = value;
                //UpdateProcessorStatsLabel();
            }
        }

        public string MedianProcessorUsageString
        {
            get
            {
                return _MedianProcessorUsageString;
            }
            set
            {
                _MedianProcessorUsageString = value;
                //UpdateProcessorStatsLabel();
            }
        }

        public bool HasRegisteredError
        {
            get
            {
                return _HasRegisteredError;
            }
            set
            {
                _HasRegisteredError = value;
            }
        }

        public EngineDisplayStatus DisplayStatus
        {
            get
            {
                return _DisplayStatus;
            }
            set
            {
                _DisplayStatus = value;
                switch (_DisplayStatus)
                {
                    case EngineDisplayStatus.Finished:
                        //pnlControls1.BackColor = Color.FromArgb(0, 180, 0);
                        // use blue instead of green for success for two reasons:
                        // 1 - to differentiate from the progress bar (on Vista/7)
                        // 2 - for people who are red/green colourblind
                        pnlControls1.BackColor = Color.FromArgb(0, 0, 180);
                        lblDirectoryName.ForeColor = Color.White;
                        lblImagesProcessed.ForeColor = Color.White;
                        break;
                    case EngineDisplayStatus.Failed:
                        pnlControls1.BackColor = Color.FromArgb(180, 0, 0);
                        lblDirectoryName.ForeColor = Color.White;
                        lblImagesProcessed.ForeColor = Color.White;
                        HasRegisteredError = true;
                        break;
                    case EngineDisplayStatus.Paused:
                        pnlControls1.BackColor = Color.FromArgb(128, 128, 0);
                        lblDirectoryName.ForeColor = Color.White;
                        lblImagesProcessed.ForeColor = Color.White;
                        break;
                    default:
                        pnlControls1.BackColor = SystemColors.Control;
                        lblDirectoryName.ForeColor = SystemColors.ControlText;
                        lblImagesProcessed.ForeColor = SystemColors.ControlText;
                        break;
                }
            }
        }

        public bool DisplayPerformanceData
        {
            get
            {
                return _DisplayPerformanceData;
            }
            set
            {
                _DisplayPerformanceData = value;
            }
        }

        public bool DisplayThumbnails
        {
            get
            {
                return _DisplayThumbnails;
            }
            set
            {
                bool valueChanged = false;
                if (value != _DisplayThumbnails)
                {
                    valueChanged = true;
                }
                _DisplayThumbnails = value;
                if (valueChanged)
                {
                    if (_DisplayThumbnails)
                    {
                        stThumbnails.Enabled = true;
                        stThumbnails.Visible = true;
                        //this.MinimumSize.Height += stThumbnails.Height;
                        this.MinimumSize = new Size(this.MinimumSize.Width, this.MinimumSize.Height + stThumbnails.Height);
                        this.Height += stThumbnails.Height;
                        pnlControls1.Height += stThumbnails.Height;
                    }
                    else
                    {
                        stThumbnails.Enabled = false;
                        stThumbnails.Visible = false;
                        pnlControls1.Height -= stThumbnails.Height;
                        //this.MinimumSize.Height -= stThumbnails.Height;
                        this.MinimumSize = new Size(this.MinimumSize.Width, this.MinimumSize.Height - stThumbnails.Height);
                        this.Height -= stThumbnails.Height;
                    }
                }
            }
        }

        #endregion

        public EngineStatusDisplay()
        {
            InitializeComponent();

            _Disposed = false;

            //pbProgress.ForeColor = Color.BlueViolet;
            //pbProgress.BackColor = Color.BlueViolet;

            lblStatus1.Text = "";
            lblStatus2.Text = "";

            _Status1Index = 0;
            _Status2Index = 0;
            _DisplayThumbnails = true;
            _DisplayPerformanceData = true;

            _StatusTimer = new Timer();
            _StatusTimer.Interval = _StatusTimerInterval;
            _StatusTimer.Tick += _StatusTimer_Tick;

            ResetState();
        }

        //~EngineStatusDisplay()
        //{
        //    this.Dispose();
        //}

        protected override void Dispose(bool disposing)
        {
            // If you need thread safety, use a lock around these 
            // operations, as well as in your methods that use the resource.
            if (!_Disposed)
            {
                //if (disposing)
                //{
                    if (_StatusTimer != null)
                    {
                        _StatusTimer.Tick -= _StatusTimer_Tick;
                        try
                        {
                            _StatusTimer.Dispose();
                            _StatusTimer = null;
                        }
                        catch (Exception ex)
                        {
                            // do nothing
                        }
                    }
                    DisposeChildControls(pnlControls1);
                    DisposeChildControls(pnlControls2);
                    if (this.stThumbnails != null)
                    {
                        try
                        {
                            this.stThumbnails.Dispose();
                            this.Controls.Remove(this.stThumbnails);
                            this.stThumbnails = null;
                        }
                        catch (Exception ex)
                        {
                            // do nothing
                        }
                    }

                    _DirectoryName = "";
                    _CurrentStatus = "";
                    _EstimatedTimeRemainingString = "";
                    _CurrentMemoryUsageString = "";
                    _MaxMemoryUsageString = "";
                    _AverageMemoryUsageString = "";
                    _MedianMemoryUsageString = "";
                    _CurrentProcessorUsageString = "";
                    _MaxProcessorUsageString = "";
                    _AverageProcessorUsageString = "";
                    _MedianProcessorUsageString = "";

                    try
                    {
                        this.Events.Dispose();
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine("Error disposing of delegate list: " + ex.Message);
                        // do nothing
                    }
                //}

                _Disposed = true;
            }
        }

        protected void DisposeChildControls(Control parentControl)
        {
            for (int i = 0; i < parentControl.Controls.Count; i++)
            {
                try
                {
                    Control currentControl = parentControl.Controls[i];
                    currentControl.Dispose();
                    parentControl.Controls.Remove(currentControl);
                    currentControl = null;
                }
                catch (Exception ex)
                {
                    // do nothing
                }
            }
        }


        public void ResetState()
        {
            DirectoryName = "Unassigned";

            pbProgress.Enabled = true;

            _CurrentStatus = "Initializing";

            _TotalImages = 0;
            _ProcessedImages = 0;
            UpdateImagesProcessedLabel();

            _EstimatedTimeRemainingString = "Waiting For Data...";
            _EstimatedTimeRemainingConfidence = 1.0f;
            //UpdateTimeRemainingLabel();

            //_CurrentMemoryUsageString = "0 B";
            //_MaxMemoryUsageString = "0 B";
            //_AverageMemoryUsageString = "0 B";
            //_MedianMemoryUsageString = "0 B";
            _CurrentMemoryUsageString = "";
            _MaxMemoryUsageString = "";
            _AverageMemoryUsageString = "";
            _MedianMemoryUsageString = "";
            //UpdateMemoryStatsLabel();

            //_CurrentProcessorUsageString = "0%";
            //_MaxProcessorUsageString = "0%";
            //_AverageProcessorUsageString = "0%";
            //_MedianProcessorUsageString = "0%";
            _CurrentProcessorUsageString = "";
            _MaxProcessorUsageString = "";
            _AverageProcessorUsageString = "";
            _MedianProcessorUsageString = "";
            //UpdateProcessorStatsLabel();

            ProcessCount = 0;

            HasRegisteredError = false;
            DisplayStatus = EngineDisplayStatus.Normal;
            _StatusTimer.Enabled = true;
        }

        protected void UpdateImagesProcessedLabel()
        {
            lblImagesProcessed.Text = "Images Processed: " + _ProcessedImages.ToString() + " / " + _TotalImages.ToString();
            pbProgress.Maximum = _TotalImages;
            pbProgress.Value = _ProcessedImages;
            this.Invalidate();
        }

        //protected void UpdateTimeRemainingLabel()
        //{
        //    lblStatus1.Text = "Estimated Time Remaining: " + _EstimatedTimeRemainingString;
        //    // use of the confidence value to vary the label colour would go here
        //    this.Invalidate();
        //}

        //protected void UpdateMemoryStatsLabel()
        //{
        //    //Memory Use: 47 MiB [Current] / 512MiB [Max] / 298 MiB [Avg] / 256MiB [Med]
        //    StringBuilder builder = new StringBuilder();
        //    builder.Append("Memory Use: ");
        //    builder.Append(_CurrentMemoryUsageString);
        //    builder.Append(" [Last] / ");
        //    builder.Append(_MaxMemoryUsageString);
        //    builder.Append(" [Max] / ");
        //    builder.Append(_AverageMemoryUsageString);
        //    builder.Append(" [Avg] / ");
        //    builder.Append(_MedianMemoryUsageString);
        //    builder.Append(" [Med]");

        //    lblStatus2.Text = builder.ToString();
        //    this.Invalidate();
        //}

        protected string GetMemoryStatsText()
        {
            if ((_CurrentMemoryUsageString != "") &&
                (_MaxMemoryUsageString != "") &&
                (_AverageMemoryUsageString != "") &&
                (_MedianMemoryUsageString != ""))
            {
                StringBuilder builder = new StringBuilder();
                builder.Append("Memory Use: ");
                builder.Append(_CurrentMemoryUsageString);
                builder.Append(" [Last] / ");
                builder.Append(_MaxMemoryUsageString);
                builder.Append(" [Max] / ");
                builder.Append(_AverageMemoryUsageString);
                builder.Append(" [Avg] / ");
                builder.Append(_MedianMemoryUsageString);
                builder.Append(" [Med]");

                return builder.ToString();
            }
            else
            {
                return "";
            }
        }

        //protected void UpdateProcessorStatsLabel()
        //{
        //    //Processor Use: 25% [Current] / 100% [Max] / 50% [Avg] / 47% [Med]
        //    StringBuilder builder = new StringBuilder();
        //    builder.Append("Processor Use: ");
        //    builder.Append(_CurrentProcessorUsageString);
        //    builder.Append(" [Last] / ");
        //    builder.Append(_MaxProcessorUsageString);
        //    builder.Append(" [Max] / ");
        //    builder.Append(_AverageProcessorUsageString);
        //    builder.Append(" [Avg] / ");
        //    builder.Append(_MedianProcessorUsageString);
        //    builder.Append(" [Med]");

        //    lblProcessorStats.Text = builder.ToString();
        //    this.Invalidate();
        //}

        protected string GetProcessorStatsText()
        {
            if ((_CurrentProcessorUsageString != "") &&
                (_MaxProcessorUsageString != "") &&
                (_AverageProcessorUsageString != "") &&
                (_MedianProcessorUsageString != ""))
            {
                StringBuilder builder = new StringBuilder();
                builder.Append("Processor Use: ");
                builder.Append(_CurrentProcessorUsageString);
                builder.Append(" [Last] / ");
                builder.Append(_MaxProcessorUsageString);
                builder.Append(" [Max] / ");
                builder.Append(_AverageProcessorUsageString);
                builder.Append(" [Avg] / ");
                builder.Append(_MedianProcessorUsageString);
                builder.Append(" [Med]");

                return builder.ToString();
            }
            else
            {
                return "";
            }
        }

        protected void _StatusTimer_Tick(object sender, EventArgs e)
        {
            StatusDisplayUpdate();
        }

        delegate void StatusDisplayUpdateCallback();

        protected void StatusDisplayUpdate()
        {
            if (this.InvokeRequired)
            {
                StatusDisplayUpdateCallback c = new StatusDisplayUpdateCallback(StatusDisplayUpdate);
                this.Invoke(c);
            }
            else
            {
                // status 1 = estimated time remaining, current status
                // status 2 = stats: processes in use, memory, CPU
                ArrayList statusText = new ArrayList();
                if ((CurrentStatus != null) && (CurrentStatus != ""))
                {
                    statusText.Add("Status: " + CurrentStatus);
                }
                if ((EstimatedTimeRemainingString != null) && (EstimatedTimeRemainingString != ""))
                {
                    // Time estimation is too buggy for now
                    //statusText.Add("Estimated Time Remaining: " + EstimatedTimeRemainingString);
                }
                if (statusText.Count > 0)
                {
                    if (_Status1Index >= statusText.Count)
                    {
                        _Status1Index = 0;
                    }
                    lblStatus1.Text = statusText[_Status1Index].ToString();
                    _Status1Index++;
                }

                statusText = new ArrayList();
                statusText.Add("Process Count: " + ProcessCount);
                if (DisplayPerformanceData)
                {
                    statusText.Add(GetMemoryStatsText());
                    statusText.Add(GetProcessorStatsText());
                }
                if (statusText.Count > 0)
                {
                    if (_Status2Index >= statusText.Count)
                    {
                        _Status2Index = 0;
                    }
                    lblStatus2.Text = statusText[_Status2Index].ToString();
                    _Status2Index++;
                }
                this.Invalidate();

                statusText.Clear();
                statusText = null;
            }
        }

        public void QueueThumbnail(string[] imagePaths, int primaryImageIndex, int thumbnailSourceImageIndex)
        {
            try
            {
                stThumbnails.QueueThumbnail(imagePaths, primaryImageIndex, thumbnailSourceImageIndex);
            }
            catch (Exception ex)
            {
                // do nothing
            }
        }

        public void QueueThumbnail(string[] imagePaths, string primaryImagePath, string thumbnailSourceImagePath)
        {
            try
            {
                stThumbnails.QueueThumbnail(imagePaths, primaryImagePath, thumbnailSourceImagePath);
            }
            catch (Exception ex)
            {
                // do nothing
            }
        }

        public void SetCompleted()
        {
            pbProgress.Value = pbProgress.Minimum;
            pbProgress.Enabled = false;
            EstimatedTimeRemainingString = "";
            //_StatusTimer.Enabled = false;
        }

        public void UpdateState(TMSBEngineState engineState)
        {
            if (HasRegisteredError)
            {
                DisplayStatus = EngineStatusDisplay.EngineDisplayStatus.Failed;
            }
            else
            {
                switch (engineState)
                {
                    case TMSBEngineState.Running:
                        DisplayStatus = EngineStatusDisplay.EngineDisplayStatus.Normal;
                        break;
                    case TMSBEngineState.Paused:
                        DisplayStatus = EngineStatusDisplay.EngineDisplayStatus.Paused;
                        break;
                    case TMSBEngineState.Finished:
                        DisplayStatus = EngineDisplayStatus.Finished;
                        break;
                    //case TMSBEngineState.Stopped:
                    //    DisplayStatus == EngineDisplayStatus.Finished;
                    //    break;
                }
            }
        }
    }
}
