// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class CustomThreeChannelImageConfig
    {
        protected string _UserDefinedName;
        protected string _RedChannelName;
        protected string _GreenChannelName;
        protected string _BlueChannelName;
        protected GreyscaleChannel _RedChannel;
        protected GreyscaleChannel _GreenChannel;
        protected GreyscaleChannel _BlueChannel;
        protected bool _Generate;
        //protected NormalizationMode _OutputNormalization;
        protected TransformationProfileCollection _InputTransformations;
        protected TransformationProfileCollection _OutputTransformations;
        protected bool _IncludeInSecondOrderImages;

        #region Properties

        public string UserDefinedName
        {
            get
            {
                return _UserDefinedName;
            }
            set
            {
                _UserDefinedName = value;
            }
        }

        public string RedChannelName
        {
            get
            {
                return _RedChannelName;
            }
            set
            {
                _RedChannelName = value;
            }
        }

        public string GreenChannelName
        {
            get
            {
                return _GreenChannelName;
            }
            set
            {
                _GreenChannelName = value;
            }
        }

        public string BlueChannelName
        {
            get
            {
                return _BlueChannelName;
            }
            set
            {
                _BlueChannelName = value;
            }
        }

        public GreyscaleChannel RedChannel
        {
            get
            {
                return _RedChannel;
            }
            set
            {
                _RedChannel = value;
            }
        }

        public GreyscaleChannel GreenChannel
        {
            get
            {
                return _GreenChannel;
            }
            set
            {
                _GreenChannel = value;
            }
        }

        public GreyscaleChannel BlueChannel
        {
            get
            {
                return _BlueChannel;
            }
            set
            {
                _BlueChannel = value;
            }
        }

        public bool Generate
        {
            get
            {
                return _Generate;
            }
            set
            {
                _Generate = value;
            }
        }

        //public NormalizationMode OutputNormalization
        //{
        //    get
        //    {
        //        return _OutputNormalization;
        //    }
        //    set
        //    {
        //        _OutputNormalization = value;
        //    }
        //}

        public TransformationProfileCollection InputTransformations
        {
            get
            {
                return _InputTransformations;
            }
            set
            {
                _InputTransformations = value;
            }
        }

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        public bool IncludeInSecondOrderImages
        {
            get
            {
                return _IncludeInSecondOrderImages;
            }
            set
            {
                _IncludeInSecondOrderImages = value;
            }
        }

        #endregion

        public CustomThreeChannelImageConfig()
        {
            _UserDefinedName = "";
            _RedChannelName = "";
            _GreenChannelName = "";
            _BlueChannelName = "";
            _Generate = true;
            //_OutputNormalization = NormalizationMode.None;
            InputTransformations = new TransformationProfileCollection();
            OutputTransformations = new TransformationProfileCollection();
            _IncludeInSecondOrderImages = true;
        }

        public void GetChannelsFromImageCube(ImageCube cube)
        {
            if (cube.ContainsGreyscaleChannel(RedChannelName))
            {
                RedChannel = cube.GetGreyscaleChannel(RedChannelName);
            }
            else
            {
                Generate = false;
                cube.CreateDebugMessage("Could not find the channel '" + RedChannelName + "' in the image cube - the custom three-channel image which references this channel will not be created.");
                return;
            }
            if (cube.ContainsGreyscaleChannel(GreenChannelName))
            {
                GreenChannel = cube.GetGreyscaleChannel(GreenChannelName);
            }
            else
            {
                Generate = false;
                cube.CreateDebugMessage("Could not find the channel '" + GreenChannelName + "' in the image cube - the custom three-channel image which references this channel will not be created.");
                return;
            }
            if (cube.ContainsGreyscaleChannel(BlueChannelName))
            {
                BlueChannel = cube.GetGreyscaleChannel(BlueChannelName);
            }
            else
            {
                Generate = false;
                cube.CreateDebugMessage("Could not find the channel '" + BlueChannelName + "' in the image cube - the custom three-channel image which references this channel will not be created.");
                return;
            }
        }
    }
}
