// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public struct GradientColourXMLParameters
    {
        public int Position;
        public ColourXMLParameters ColourParameters;
    }

    public class GradientColour : Colour, IComparable
    {
        protected int _Position;

        #region Properties

        public int Position
        {
            get
            {
                return _Position;
            }
            set
            {
                if ((value < 1) || (value > 65536))
                {
                    throw new ArgumentOutOfRangeException("The position for a gradient colour must be between 1 and 65536.");
                }
                _Position = value;
            }
        }

        #endregion

        public GradientColour()
            : base()
        {
            _Position = 1;
        }

        public int CompareTo(object obj)
        {
            if (obj is GradientColour)
            {
                GradientColour b2 = (GradientColour)obj;
                return Position.CompareTo(b2.Position);
            }
            return Position.CompareTo(obj.ToString());
        }


        public static GradientColour FromXMLParameters(GradientColourXMLParameters xmlParams)
        {
            GradientColour newGC = new GradientColour();
            try
            {
                Colour.RGBValueFormat vFormat = Colour.RGBValueFormatFromString(xmlParams.ColourParameters.Format);
                switch (vFormat)
                {
                    case Colour.RGBValueFormat.Byte8:
                        //GradientColour newGC = From8BitRGB((int)xmlParams.ColourParameters.Red, (int)xmlParams.ColourParameters.Green,
                        //    (int)xmlParams.ColourParameters.Blue);
                        Colour c = From8BitRGB((int)xmlParams.ColourParameters.Red, (int)xmlParams.ColourParameters.Green,
                            (int)xmlParams.ColourParameters.Blue);
                        newGC.Red = c.Red;
                        newGC.Green = c.Green;
                        newGC.Blue = c.Blue;
                        newGC.Position = xmlParams.Position;
                        return newGC;
                        break;
                    case Colour.RGBValueFormat.UInt16:
                        Colour c2 = From16BitRGB((int)xmlParams.ColourParameters.Red, (int)xmlParams.ColourParameters.Green,
                            (int)xmlParams.ColourParameters.Blue);
                        newGC.Red = c2.Red;
                        newGC.Green = c2.Green;
                        newGC.Blue = c2.Blue;
                        newGC.Position = xmlParams.Position;
                        return newGC;
                        break;
                    case Colour.RGBValueFormat.Float32:
                        newGC.Red = xmlParams.ColourParameters.Red;
                        newGC.Green = xmlParams.ColourParameters.Green;
                        newGC.Blue = xmlParams.ColourParameters.Blue;
                        newGC.Position = xmlParams.Position;
                        return newGC;
                        break;
                }
            }
            catch (ArgumentException argEx)
            {
                throw new ArgumentException("Could not create a gradient colour from the passed XML parameters. Verify that the Format value is valid.", argEx);
            }
            catch (Exception ex)
            {
                throw new ArgumentException("Could not create a gradient colour from the passed XML parameters.", ex);
            }
            throw new ArgumentException("An unexpected result occurred attempting to create a gradient colour. Please verify that the XML definition conforms to the required specifications.");
        }
    }
}
