// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

using BenLincoln.Utility;

namespace BenLincoln.DaVinci
{
    public class ImageFinishedEventArgs : ImageEventArgs
    {
        protected TimeSpan _ElapsedTime;
        protected NumericValueTracker _MemoryUseTracker;
        protected NumericValueTracker _ProcessorTimeTracker;
        protected int _GlobalImageNumber;
        protected string _BaseDirectory;
        protected string[] _OutputImages;

        #region properties

        public TimeSpan ElapsedTime
        {
            get
            {
                return _ElapsedTime;
            }
        }

        public NumericValueTracker MemoryUseTracker
        {
            get
            {
                return _MemoryUseTracker;
            }
        }

        public NumericValueTracker ProcessorTimeTracker
        {
            get
            {
                return _ProcessorTimeTracker;
            }
        }

        public int GlobalImageNumber
        {
            get
            {
                return _GlobalImageNumber;
            }
        }

        public string BaseDirectory
        {
            get
            {
                return _BaseDirectory;
            }
        }

        public string[] OutputImages
        {
            get
            {
                return _OutputImages;
            }
        }

        #endregion

        public ImageFinishedEventArgs(int engineNumber, int globalThreadNumber, string sectionName, string imageName, int imageNumber, 
            int totalImages, int totalImagesInDirectory, TimeSpan elapsedTime, NumericValueTracker memoryUseTracker, NumericValueTracker processorTimeTracker, 
            int globalImageNumber, string baseDirectory, ScriptQuantum quantum)
            : base(engineNumber, globalThreadNumber, sectionName, imageName, imageNumber, totalImages, totalImagesInDirectory)
        {
            _ElapsedTime = elapsedTime;
            _MemoryUseTracker = memoryUseTracker;
            _ProcessorTimeTracker = processorTimeTracker;
            _GlobalImageNumber = globalImageNumber;
            _OutputImages = new string[quantum.OutputImages.Length];
            _BaseDirectory = baseDirectory;
            for (int i = 0; i < _OutputImages.Length; i++)
            {
                _OutputImages[i] = baseDirectory + Path.DirectorySeparatorChar + quantum.OutputImages[i];
            }
        }
    }
}
