// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Threading;

namespace BenLincoln.DaVinci
{
    public class ProcessOutputHandler
    {
        protected Process _Process;
        protected string _StandardOutput;
        protected string _StandardError;

        protected const int _SleepLength = 1;

        #region Properties

        public string StandardOutput
        {
            get
            {
                return _StandardOutput;
            }
        }

        public string StandardError
        {
            get
            {
                return _StandardError;
            }
        }

        #endregion

        public ProcessOutputHandler(Process proc)
        {
            _Process = proc;
        }

        ~ProcessOutputHandler()
        {
            _Process.Dispose();
        }

        public void StartStandardOutputRead()
        {
            try
            {
                StringBuilder builder = new StringBuilder();
                while ((!_Process.HasExited) && (!_Process.StandardOutput.EndOfStream))
                {
                    if (!_Process.StandardOutput.EndOfStream)
                    {
                        builder.AppendLine(_Process.StandardOutput.ReadLine());
                    }
                    else
                    {
                        Thread.Sleep(_SleepLength);
                    }
                }
                _StandardOutput = builder.ToString();
            }
            catch (Exception ex)
            {
                //throw new Exception("Unable to begin reading the process' standard output. The error description was '" + ex.Message + "'.");
            }
        }

        public void StartStandardErrorRead()
        {
            try
            {
                StringBuilder builder = new StringBuilder();
                while ((!_Process.HasExited) && (!_Process.StandardError.EndOfStream))
                {
                    if (!_Process.StandardError.EndOfStream)
                    {
                        builder.AppendLine(_Process.StandardError.ReadLine());
                    }
                    else
                    {
                        Thread.Sleep(_SleepLength);
                    }
                }
                _StandardError = builder.ToString();
            }
            catch (Exception ex)
            {
                //throw new Exception("Unable to begin reading the process' standard error output. The error description was '" + ex.Message + "'.");
            }
        }
    }
}
