using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class ScriptVariableCollection : CollectionBase
    {

        public ScriptVariableCollection()
        {
        }

        public ScriptVariableCollection(ScriptVariableCollection sourceCollection)
            : this()
        {
            for (int i = 0; i < sourceCollection.Count; i++)
            {
                this.Add(new ScriptVariable(sourceCollection[i]));
            }
        }

        public ScriptVariable this[int index]
        {
            get
            {
                return (ScriptVariable)this.List[index];
            }
            set
            {
                int ind = IndexOf(value);
                if ((ind > -1) && (ind != index))
                {
                    throw new Exception("The collection already contains a variable named '" + value.Name + "'.");
                }
                this.List[index] = value;
            }
        }

        public ScriptVariable this[string svName]
        {
            get
            {
                int index = IndexOf(svName);
                if (index == -1)
                {
                    throw new KeyNotFoundException("No script variable with the name '" + svName + "' was found in the collection.");
                }
                return this[index];
            }
            set
            {
                int index = IndexOf(svName);
                if (index == -1)
                {
                    throw new KeyNotFoundException("No script variable with the name '" + svName + "' was found in the collection.");
                }
                this[index] = value;
            }
        }

        public int IndexOf(ScriptVariable sv)
        {
            return IndexOf(sv.Name);
        }

        public int IndexOf(string svName)
        {
            for (int i = 0; i < this.Count; i++)
            {
                if (this[i].Name == svName)
                {
                    return i;
                }
            }
            return -1;
        }

        public int Add(ScriptVariable sv)
        {
            if (Contains(sv))
            {
                throw new Exception("The collection already contains a variable named '" + sv.Name + "'.");
            }
            return base.List.Add(sv);
        }

        public bool Contains(string svName)
        {
            if (IndexOf(svName) > -1)
            {
                return true;
            }
            return false;
        }

        public bool Contains(ScriptVariable sv)
        {
            return Contains(sv.Name);
        }

        public void Remove(ScriptVariable sv)
        {
            this.List.Remove(sv);
        }

        public void CopyTo(Array array, int index)
        {
            this.List.CopyTo(array, index);
        }

        public void AddRange(ScriptVariableCollection coll)
        {
            for (int i = 0; i < coll.Count; i++)
            {
                this.List.Add(coll[i]);
            }
        }

        public void AddRange(ScriptVariable[] coll)
        {
            foreach (ScriptVariable sv in coll)
            {
                this.List.Add(sv);
            }
        }

        public void Insert(int index, ScriptVariable sv)
        {
            this.List.Insert(index, sv);
        }

        public void Sort()
        {
            InnerList.Sort();
        }
    }
}
