// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class SpectralBandData
    {
        protected string _FileName;
        protected string _Plane;
        protected const string _PlaneStringTemplate = "[0,0,x]";

        #region Properties

        public string FileName
        {
            get
            {
                return _FileName;
            }
            set
            {
                _FileName = value;
            }
        }

        public string Plane
        {
            get
            {
                return _Plane;
            }
            set
            {
                _Plane = value;
            }
        }

        public int PlaneNumber
        {
            get
            {
                return GetPlaneNumber();
            }
            set
            {
                _Plane = GetFormattedPlaneString(value);
            }
        }

        #endregion

        public SpectralBandData()
        {
            _FileName = "";
            _Plane = "";
        }

        public SpectralBandData(string fileName, string plane)
            : this()
        {
            _FileName = fileName;
            _Plane = plane;
        }

        public SpectralBandData(string fileName, int planeNumber)
            : this(fileName, GetFormattedPlaneString(planeNumber))
        {
        }

        // these two could be made better if fancier DaVinci image cube handling is 
        // ever implemented.
        public static string GetFormattedPlaneString(int planeNumber)
        {
            return _PlaneStringTemplate.Replace("x", BenLincoln.Utility.FormatConverter.IntToString(planeNumber));
        }

        public int GetPlaneNumber()
        {
            int prefixLength = _PlaneStringTemplate.IndexOf("x") - 1;
            int suffixLength = _PlaneStringTemplate.Length - (prefixLength + 1);
            int valueLength = _PlaneStringTemplate.Length - suffixLength;
            string planeString = Plane.Substring(prefixLength, valueLength);
            return BenLincoln.Utility.FormatConverter.StringToInt(planeString);
        }
    }
}
