// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public struct SpectralBandWithFlag
    {
        public SpectralBand Band;
        public bool Flag;
    }

    public class SpectralBandFlagList
    {
        protected ArrayList _SpectralBands;

        public SpectralBandFlagList()
        {
            _SpectralBands = new ArrayList();
        }

        public void AddBand(SpectralBand newBand)
        {
            SpectralBandWithFlag bwf = new SpectralBandWithFlag();
            bwf.Band = newBand;
            if (_SpectralBands.Count == 0)
            {
                bwf.Flag = true;
            }
            else
            {
                bwf.Flag = false;
            }
            _SpectralBands.Add(bwf);
        }

        public SpectralBandWithFlag GetBand(int bandNumber)
        {
            SpectralBandWithFlag bwf = (SpectralBandWithFlag)_SpectralBands[bandNumber];
            return bwf;
        }

        public int GetLength()
        {
            return _SpectralBands.Count;
        }

        public void ResetFlags()
        {
            if (_SpectralBands.Count == 0)
            {
                return;
            }
            SpectralBandWithFlag bwf = (SpectralBandWithFlag)_SpectralBands[0];
            bwf.Flag = true;
            _SpectralBands[0] = bwf;
            if (_SpectralBands.Count == 1)
            {
                return;
            }
            for (int i = 1; i < _SpectralBands.Count; i++)
            {
                bwf = (SpectralBandWithFlag)_SpectralBands[i];
                bwf.Flag = false;
                _SpectralBands[i] = bwf;
            }
        }

        public void IncrementFlags()
        {
            bool incrementNext = true;
            for (int i = (_SpectralBands.Count - 1); i >= 0; i--)
            {
                if (incrementNext == true)
                {
                    SpectralBandWithFlag bwf = (SpectralBandWithFlag)_SpectralBands[i];
                    if (bwf.Flag == false)
                    {
                        bwf.Flag = true;
                        incrementNext = false;
                    }
                    else
                    {
                        bwf.Flag = false;
                    }
                    _SpectralBands[i] = bwf;
                }
            }
        }


    }
}
