// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class StatisticalCubeConfig
    {
        protected string _CubeName;
        protected int _WindowSize;
        protected bool _UseAsThreeChannelChannels;
        protected CombinedBandConfig[] _CombinedBandConfigurations;
        protected TransformationProfileCollection _InputTransformations;
        protected TransformationProfileCollection _OutputTransformations;

        #region Properties

        public string CubeName
        {
            get
            {
                return _CubeName;
            }
            set
            {
                _CubeName = value;
            }
        }

        public int WindowSize
        {
            get
            {
                return _WindowSize;
            }
            set
            {
                _WindowSize = value;
            }
        }

        public bool UseAsThreeChannelChannels
        {
            get
            {
                return _UseAsThreeChannelChannels;
            }
            set
            {
                _UseAsThreeChannelChannels = value;
            }
        }

        public CombinedBandConfig[] CombinedBandConfigurations
        {
            get
            {
                return _CombinedBandConfigurations;
            }
        }

        public TransformationProfileCollection InputTransformations
        {
            get
            {
                return _InputTransformations;
            }
            set
            {
                _InputTransformations = value;
            }
        }

        public TransformationProfileCollection OutputTransformations
        {
            get
            {
                return _OutputTransformations;
            }
            set
            {
                _OutputTransformations = value;
            }
        }

        #endregion

        public StatisticalCubeConfig()
        {
            _CombinedBandConfigurations = new CombinedBandConfig[0];
            UseAsThreeChannelChannels = false;
            CubeName = "";
            WindowSize = 1;
            InputTransformations = new TransformationProfileCollection();
            OutputTransformations = new TransformationProfileCollection();
        }

        public StatisticalCubeConfig(string cubeName, int windowSize)
            : this()
        {
            CubeName = cubeName;
            WindowSize = windowSize;
        }

        public bool ContainsBandCombineMethod(BandCombineMethod method)
        {
            return CombinedBandConfig.ArrayContainsBandCombineMethod(_CombinedBandConfigurations, method);
        }

        public void AddStatisticalGreyscaleConfiguration(CombinedBandConfig newConfig)
        {
            if (ContainsBandCombineMethod(newConfig.BandCombineMode))
            {
                throw new ArgumentException("An attempt was made to add an additional statistical greyscale variation based on the '" + newConfig.BandCombineMode.ToString() + "' method. Multiple variations using the same combination method are not currently supported.");
            }
            Array.Resize(ref _CombinedBandConfigurations, _CombinedBandConfigurations.Length + 1);
            _CombinedBandConfigurations[_CombinedBandConfigurations.Length - 1] = newConfig;
        }
    }
}
