// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class TMSBColourConfiguration
    {
        protected TintingColour[] _TintingColours;

        protected ColourGradient[] _ColourGradients;

        #region Properties

        public TintingColour[] TintingColours
        {
            get
            {
                return _TintingColours;
            }
            set
            {
                _TintingColours = value;
            }
        }

        public ColourGradient[] ColourGradients
        {
            get
            {
                return _ColourGradients;
            }
            set
            {
                _ColourGradients = value;
            }
        }

        #endregion

        public TMSBColourConfiguration()
        {
            TintingColours = new TintingColour[0];

            ColourGradients = new ColourGradient[0];
        }

        public void AddTintingColour(TintingColour newTintingColour)
        {
            Array.Resize(ref _TintingColours, _TintingColours.Length + 1);
            _TintingColours[_TintingColours.Length - 1] = newTintingColour;
        }

        public void AddColourGradient(ColourGradient newColourGradient)
        {
            Array.Resize(ref _ColourGradients, _ColourGradients.Length + 1);
            ColourGradients[ColourGradients.Length - 1] = newColourGradient;
        }


        public bool ContainsTintingColour(string colourName)
        {
            foreach (TintingColour colour in TintingColours)
            {
                if (colour.TintingColourName == colourName)
                {
                    return true;
                }
            }
            return false;
        }

        public TintingColour GetTintingColour(string colourName)
        {
            foreach (TintingColour colour in TintingColours)
            {
                if (colour.TintingColourName == colourName)
                {
                    return colour;
                }
            }
            throw new KeyNotFoundException("The colour configuration does not contain a tinting colour named '" + colourName + "'.");
        }

        public bool ContainsGradient(string gradientName)
        {
            foreach (ColourGradient grad in ColourGradients)
            {
                if (grad.GradientName == gradientName)
                {
                    return true;
                }
            }
            return false;
        }

        public ColourGradient GetGradient(string gradientName)
        {
            foreach (ColourGradient grad in ColourGradients)
            {
                if (grad.GradientName == gradientName)
                {
                    return grad;
                }
            }
            throw new KeyNotFoundException("The colour configuration does not contain a gradient named '" + gradientName + "'.");
        }
    }
}
