// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

namespace BenLincoln.DaVinci
{
    public class TMSBEngineControllerConfigurationFile : TMSBConfigurationFile
    {
        protected TMSBEngineControllerConfiguration _EngineControllerConfig;

        #region Properties

        public TMSBEngineControllerConfiguration EngineControllerConfig
        {
            get
            {
                return _EngineControllerConfig;
            }
            set
            {
                _EngineControllerConfig = value;
            }
        }

        #endregion

        protected enum ConfigProcessingState
        {
            Normal
        }

        public TMSBEngineControllerConfigurationFile(string configFileName)
            : base()
        {
            _EngineControllerConfig = new TMSBEngineControllerConfiguration();
            ReadData(configFileName);
        }

        protected override void ReadTypeSpecificData(Stream sourceStream)
        {
            XmlTextReader xReader = new XmlTextReader(sourceStream);
            //xReader.Settings.Schemas.Add(_FileSchema);

            string currentElementName = "";
            ConfigProcessingState currentState = ConfigProcessingState.Normal;
            while (xReader.Read())
            {
                switch (xReader.NodeType)
                {
                    case XmlNodeType.Element:
                        currentElementName = xReader.Name;
                        //switch (currentState)
                        //{
                        //    case ConfigProcessingState.Normal:
                        //        if (currentElementName == "")
                        //        {
                        //            currentState = ConfigProcessingState.FormatSection;
                        //        }
                        //        break;
                        //}
                        break;
                    case XmlNodeType.EndElement:
                        currentElementName = xReader.Name;
                        //switch (currentState)
                        //{
                        //    case ConfigProcessingState.FormatSection:
                        //        if (currentElementName == "")
                        //        {
                        //            currentState = ConfigProcessingState.Normal;
                        //        }
                        //        break;
                        //}
                        break;
                    case XmlNodeType.Text:
                        string currentText = xReader.Value.Trim();
                        switch (currentState)
                        {
                            case ConfigProcessingState.Normal:
                                switch (currentElementName)
                                {
                                    // processing criteria
                                    case "MinimumBandsInSourceDirectories":
                                        _EngineControllerConfig.MinimumSourceBandsInInputDirectory = BenLincoln.Utility.FormatConverter.StringToInt(currentText);
                                        break;
                                    case "ProcessDirectoriesRecursively":
                                        _EngineControllerConfig.ScanInputDirectoriesRecursively = StringToBool(currentText);
                                        break;

                                    // config files
                                    case "InputConfigurationFile":
                                        _EngineControllerConfig.InputConfigFilePath = currentText;
                                        break;
                                    case "ProcessingConfigurationFile":
                                        _EngineControllerConfig.ProcessingConfigFilePath = currentText;
                                        break;
                                    case "OutputConfigurationFile":
                                        _EngineControllerConfig.OutputConfigFilePath = currentText;
                                        break;

                                    // script processor
                                    case "GenerateScripts":
                                        _EngineControllerConfig.GenerateScripts = TMSBEngineControllerConfiguration.ScriptGenerationModeFromString(currentText);
                                        break;
                                    case "ExecuteScripts":
                                        _EngineControllerConfig.ExecuteScripts = StringToBool(currentText);
                                        break;
                                    case "ScriptProcessorPath":
                                        //_EngineControllerConfig.DaVinciCommand = currentText;
                                        if (Path.IsPathRooted(currentText))
                                        {
                                            _EngineControllerConfig.DaVinciCommand = currentText;
                                        }
                                        else
                                        {
                                            try
                                            {
                                                _EngineControllerConfig.DaVinciCommand = Path.GetFullPath(currentText);
                                            }
                                            catch (Exception ex)
                                            {
                                                _EngineControllerConfig.DaVinciCommand = "";
                                            }
                                        }
                                        if ((_EngineControllerConfig.DaVinciCommand != "") && (!File.Exists(_EngineControllerConfig.DaVinciCommand)))
                                        {
                                            _EngineControllerConfig.DaVinciCommand = "";
                                        }
                                        break;
                                    case "ScriptProcessorArguments":
                                        _EngineControllerConfig.DaVinciCommandArgs = currentText;
                                        break;
                                    case "ScriptProcessPriority":
                                        _EngineControllerConfig.DaVinciProcessPriority = TMSBEngineControllerConfiguration.ProcessPriorityClassFromString(currentText);
                                        break;
                                    case "MaximumSimultaneousDirectories":
                                        _EngineControllerConfig.EngineThreadCount = BenLincoln.Utility.FormatConverter.StringToInt(currentText);
                                        break;
                                    case "MaximumSimultaneousImages":
                                        _EngineControllerConfig.GlobalThreadCount = BenLincoln.Utility.FormatConverter.StringToInt(currentText);
                                        break;
                                    case "SubscriptExecutionTimeLimit":
                                        _EngineControllerConfig.ScriptExecutionTimeLimit = BenLincoln.Utility.FormatConverter.StringToInt(currentText);
                                        break;

                                    // performance stats
                                    case "CollectPerformanceData":
                                        _EngineControllerConfig.CollectPerformanceData = StringToBool(currentText);
                                        break;
                                    case "MemoryUnits":
                                        _EngineControllerConfig.MemoryUnits = TMSBEngineControllerConfiguration.MemoryRepresentationFromString(currentText);
                                        break;

                                    // debugging/troubleshooting
                                    case "LogLevel":
                                        _EngineControllerConfig.OutputLevel = TMSBEngineControllerConfiguration.OutputLevelFromString(currentText);
                                        break;
                                    case "LogPath":
                                        _EngineControllerConfig.LogFilePath = currentText;
                                        break;
                                    case "LogMode":
                                        _EngineControllerConfig.LogFileMode = TMSBEngineControllerConfiguration.LogModeFromString(currentText);
                                        break;
                                    case "IgnoreScriptErrors":
                                        _EngineControllerConfig.HaltOnErrors = !StringToBool(currentText);
                                        break;
                                    case "KeepTemporaryFiles":
                                        _EngineControllerConfig.DeleteTemporaryFiles = !StringToBool(currentText);
                                        break;
                                    case "WriteScriptCodeEvenWhenNoFilesWillBeGenerated":
                                        _EngineControllerConfig.WriteScriptCodeWhenNoImagesWillBeGenerated = StringToBool(currentText);
                                        break;
                                    case "DisplayImageThumbnails":
                                        _EngineControllerConfig.DisplayImageThumbnails = StringToBool(currentText);
                                        break;
                                    case "ExecutionPlatformMode":
                                        _EngineControllerConfig.ExecutionPlatformMode = TMSBEngineControllerConfiguration.ExecutionPlatformModeFromString(currentText);
                                        break;
                                }
                                break;
                        }
                        break;
                }
            }
        }

        protected override void WriteTypeSpecificData(XmlTextWriter oWriter)
        {
            oWriter.WriteStartElement("ExecutionOptions");
            
            // processing criteria

            oWriter.WriteStartElement("MinimumBandsInSourceDirectories");
            oWriter.WriteValue(_EngineControllerConfig.MinimumSourceBandsInInputDirectory);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("ProcessDirectoriesRecursively");
            oWriter.WriteValue(_EngineControllerConfig.ScanInputDirectoriesRecursively);
            oWriter.WriteEndElement();


            // config files

            oWriter.WriteStartElement("InputConfigurationFile");
            oWriter.WriteValue(_EngineControllerConfig.InputConfigFilePath);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("ProcessingConfigurationFile");
            oWriter.WriteValue(_EngineControllerConfig.ProcessingConfigFilePath);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("OutputConfigurationFile");
            oWriter.WriteValue(_EngineControllerConfig.OutputConfigFilePath);
            oWriter.WriteEndElement();

            // script processor

            oWriter.WriteStartElement("GenerateScripts");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.ScriptGenerationModeToString(_EngineControllerConfig.GenerateScripts));
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("ExecuteScripts");
            oWriter.WriteValue(_EngineControllerConfig.ExecuteScripts);
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("ScriptProcessorPath");
            oWriter.WriteValue(_EngineControllerConfig.DaVinciCommand);
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("ScriptProcessorArguments");
            oWriter.WriteValue(_EngineControllerConfig.DaVinciCommandArgs);
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("ScriptProcessPriority");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.ProcessPriorityClassToString(_EngineControllerConfig.DaVinciProcessPriority));
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("MaximumSimultaneousDirectories");
            oWriter.WriteValue(_EngineControllerConfig.EngineThreadCount);
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("MaximumSimultaneousImages");
            oWriter.WriteValue(_EngineControllerConfig.GlobalThreadCount);
            oWriter.WriteEndElement();
            
            oWriter.WriteStartElement("SubscriptExecutionTimeLimit");
            oWriter.WriteValue(_EngineControllerConfig.ScriptExecutionTimeLimit);
            oWriter.WriteEndElement();
            

            // performance data

            oWriter.WriteStartElement("CollectPerformanceData");
            oWriter.WriteValue(_EngineControllerConfig.CollectPerformanceData);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("MemoryUnits");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.MemoryRepresentationToString(_EngineControllerConfig.MemoryUnits));
            oWriter.WriteEndElement();

            // debugging/troubleshooting

            oWriter.WriteStartElement("LogLevel");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.OutputLevelToString(_EngineControllerConfig.OutputLevel));
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("LogPath");
            oWriter.WriteValue(_EngineControllerConfig.LogFilePath);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("LogMode");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.LogModeToString(_EngineControllerConfig.LogFileMode));
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("IgnoreScriptErrors");
            oWriter.WriteValue(!_EngineControllerConfig.HaltOnErrors);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("KeepTemporaryFiles");
            oWriter.WriteValue(!_EngineControllerConfig.DeleteTemporaryFiles);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("WriteScriptCodeEvenWhenNoFilesWillBeGenerated");
            oWriter.WriteValue(_EngineControllerConfig.WriteScriptCodeWhenNoImagesWillBeGenerated);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("DisplayImageThumbnails");
            oWriter.WriteValue(_EngineControllerConfig.DisplayImageThumbnails);
            oWriter.WriteEndElement();

            oWriter.WriteStartElement("ExecutionPlatformMode");
            oWriter.WriteValue(TMSBEngineControllerConfiguration.ExecutionPlatformModeToString(_EngineControllerConfig.ExecutionPlatformMode));
            oWriter.WriteEndElement();

            oWriter.WriteEndElement();
        }
    }
}
