// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public class TMSBProcessingConfiguration
    {
        //protected NormalizationMode _ColourImageOverallOutputNormalization;

        protected ScriptVariableCollection _ScriptVariables;
        
        // trying to set this up as a protected member with a property to access it was going to be 
        // a huge headache. I did it everywhere else, but not here.
        public TransformationProfileCollection ColourImageOutputTransformations;

        protected CombinedBandConfig[] _ChannelCombinationMethods;
        protected StatisticalCubeConfig[] _StatisticalGreyscaleCubeConfigurations;

        protected CustomGreyscaleChannelConfig[] _CustomGreyscaleChannelConfigurations;

        protected CalculatedChannelCombinationConfig[] _CalculatedChannelConfigurations;

        protected CustomThreeChannelImageConfig[] _CustomThreeChannelImageConfigurations;

        protected TintMapping[] _TintMappings;
        protected TintMapping[] _GradientMappings;

        protected bool _GenerateStatisticalImageThreeChannelPermutations;
        protected bool _GenerateStatisticalImageThreeChannelPermutationsUnrestricted;

        protected bool _GenerateStatisticalChannels;
        protected bool _GenerateCalculatedChannels;
        protected bool _GenerateCustomChannels;
        protected bool _GenerateTintedImages;
        protected bool _GenerateGradientImages;
        protected bool _GenerateThreeChannelImages;
        protected bool _GenerateStandardThreeChannelImages;
        protected bool _GenerateCustomThreeChannelImages;
        protected bool _GenerateCalculatedThreeChannelImages;
        protected bool _GenerateLumaColourImages;
        protected bool _GenerateDCSImages;

        protected bool _GenerateLumaColourVariationsFromIndividualBands;
        protected bool _GenerateLumaColourVariationsFromCombinedChannels;
        protected bool _GenerateLumaColourVariationsFromStatisticalChannels;
        protected bool _GenerateLumaColourVariationsFromThreeChannelImages;
        protected bool _GenerateLumaColourVariationsFromCustomThreeChannelImages;
        protected bool _GenerateLumaColourVariationsFromTintedImages;
        protected bool _GenerateLumaColourVariationsFromGradientMappedImages;
        protected bool _GenerateLumaColourVariationsFromDCSImages;

        protected bool _GenerateDCSImagesFromIndividualChannels;
        protected bool _GenerateDCSImagesFromCombinedChannels;
        protected bool _GenerateDCSImagesFromCustomThreeChannelImages;

        protected bool _GenerateDCSEnhancedContrast;

        protected DCSParameterSet[] _DCSTypes;

        #region Properties

        public ScriptVariableCollection ScriptVariables
        {
            get
            {
                return _ScriptVariables;
            }
            set
            {
                _ScriptVariables = value;
            }
        }

        //public NormalizationMode ColourImageOverallOutputNormalization
        //{
        //    get
        //    {
        //        return _ColourImageOverallOutputNormalization;
        //    }
        //    set
        //    {
        //        _ColourImageOverallOutputNormalization = value;
        //    }
        //}

        //public TransformationProfileCollection ColourImageOutputTransformations
        //{
        //    get
        //    {
        //        return _ColourImageOutputTransformations;
        //    }
        //    set
        //    {
        //        _ColourImageOutputTransformations = value;
        //    }
        //}

        public CombinedBandConfig[] ChannelCombinationMethods
        {
            get
            {
                return _ChannelCombinationMethods;
            }
            set
            {
                _ChannelCombinationMethods = value;
            }
        }

        public StatisticalCubeConfig[] StatisticalGreyscaleCubeConfigurations
        {
            get
            {
                return _StatisticalGreyscaleCubeConfigurations;
            }
            set
            {
                _StatisticalGreyscaleCubeConfigurations = value;
            }
        }

        public CustomGreyscaleChannelConfig[] CustomGreyscaleChannelConfigurations
        {
            get
            {
                return _CustomGreyscaleChannelConfigurations;
            }
            set
            {
                _CustomGreyscaleChannelConfigurations = value;
            }
        }

        public CalculatedChannelCombinationConfig[] CalculatedChannelConfigurations
        {
            get
            {
                return _CalculatedChannelConfigurations;
            }
            set
            {
                _CalculatedChannelConfigurations = value;
            }
        }

        public CustomThreeChannelImageConfig[] CustomThreeChannelImageConfigurations
        {
            get
            {
                return _CustomThreeChannelImageConfigurations;
            }
            set
            {
                _CustomThreeChannelImageConfigurations = value;
            }
        }

        public TintMapping[] TintMappings
        {
            get
            {
                return _TintMappings;
            }
            set
            {
                _TintMappings = value;
            }
        }

        public TintMapping[] GradientMappings
        {
            get
            {
                return _GradientMappings;
            }
            set
            {
                _GradientMappings = value;
            }
        }

        public bool GenerateStatisticalChannels
        {
            get
            {
                return _GenerateStatisticalChannels;
            }
            set
            {
                _GenerateStatisticalChannels = value;
            }
        }

        public bool GenerateCalculatedChannels
        {
            get
            {
                return _GenerateCalculatedChannels;
            }
            set
            {
                _GenerateCalculatedChannels = value;
            }
        }

        public bool GenerateCustomChannels
        {
            get
            {
                return _GenerateCustomChannels;
            }
            set
            {
                _GenerateCustomChannels = value;
            }
        }

        public bool GenerateTintedImages
        {
            get
            {
                return _GenerateTintedImages;
            }
            set
            {
                _GenerateTintedImages = value;
            }
        }

        public bool GenerateGradientImages
        {
            get
            {
                return _GenerateGradientImages;
            }
            set
            {
                _GenerateGradientImages = value;
            }
        }

        public bool GenerateThreeChannelImages
        {
            get
            {
                return _GenerateThreeChannelImages;
            }
            set
            {
                _GenerateThreeChannelImages = value;
            }
        }

        public bool GenerateStandardThreeChannelImages
        {
            get
            {
                return _GenerateStandardThreeChannelImages;
            }
            set
            {
                _GenerateStandardThreeChannelImages = value;
            }
        }

        public bool GenerateCustomThreeChannelImages
        {
            get
            {
                return _GenerateCustomThreeChannelImages;
            }
            set
            {
                _GenerateCustomThreeChannelImages = value;
            }
        }

        public bool GenerateCalculatedThreeChannelImages
        {
            get
            {
                return _GenerateCalculatedThreeChannelImages;
            }
            set
            {
                _GenerateCalculatedThreeChannelImages = value;
            }
        }

        public bool GenerateLumaColourImages
        {
            get
            {
                return _GenerateLumaColourImages;
            }
            set
            {
                _GenerateLumaColourImages = value;
            }
        }

        public bool GenerateDCSImages
        {
            get
            {
                return _GenerateDCSImages;
            }
            set
            {
                _GenerateDCSImages = value;
            }
        }

        public bool GenerateStatisticalImageThreeChannelPermutations
        {
            get
            {
                return _GenerateStatisticalImageThreeChannelPermutations;
            }
            set
            {
                _GenerateStatisticalImageThreeChannelPermutations = value;
            }
        }

        public bool GenerateStatisticalImageThreeChannelPermutationsUnrestricted
        {
            get
            {
                return _GenerateStatisticalImageThreeChannelPermutationsUnrestricted;
            }
            set
            {
                _GenerateStatisticalImageThreeChannelPermutationsUnrestricted = false;
                if (_GenerateStatisticalImageThreeChannelPermutationsUnrestricted)
                {
                    GenerateStatisticalImageThreeChannelPermutations = true;
                }
            }
        }

        /// <summary>
        /// Controls whether or not colour/luma variations will be created by using 
        /// all colour images combined with all base spectral bands.
        /// Example: NIR_Luminance-(R-G-B)_Colour
        /// </summary>
        public bool GenerateLumaColourVariationsFromIndividualBands
        {
            get
            {
                return _GenerateLumaColourVariationsFromIndividualBands;
            }
            set
            {
                _GenerateLumaColourVariationsFromIndividualBands = value;
            }
        }

        /// <summary>
        /// Controls whether or not colour/luma variations will be created by using 
        /// all colour images combined with all greyscale channels (when those greyscale 
        /// channels are not the result of statistical output). If this option is 
        /// set to false, only greyscale channels which represent a single spectral 
        /// band will be used. Setting this value to true will produce an extremely 
        /// large number of variations.
        /// Example: NIR_R_(Avg)_Luminance-(G-B-UVA)_Colour
        /// </summary>
        public bool GenerateLumaColourVariationsFromCombinedChannels
        {
            get
            {
                return _GenerateLumaColourVariationsFromCombinedChannels;
            }
            set
            {
                _GenerateLumaColourVariationsFromCombinedChannels = value;
            }
        }

        /// <summary>
        /// Controls whether or not colour/luma variations will be created by using 
        /// all colour images combined with all statistically-generated greyscale channels.
        /// Example: All_Bands_(Med)_Luminance-(G-B-UVA)_Colour
        /// </summary>
        public bool GenerateLumaColourVariationsFromStatisticalChannels
        {
            get
            {
                return _GenerateLumaColourVariationsFromStatisticalChannels;
            }
            set
            {
                _GenerateLumaColourVariationsFromStatisticalChannels = value;
            }
        }

        /// <summary>
        /// Controls whether or not colour/luma variations will be created by using 
        /// 
        /// Example: 
        /// </summary>
        public bool GenerateLumaColourVariationsFromThreeChannelImages
        {
            get
            {
                return _GenerateLumaColourVariationsFromThreeChannelImages;
            }
            set
            {
                _GenerateLumaColourVariationsFromThreeChannelImages = value;
            }
        }

        /// <summary>
        /// Controls whether or not colour/luma variations will be created by using 
        /// 
        /// Example: 
        /// </summary>
        public bool GenerateLumaColourVariationsFromDCSImages
        {
            get
            {
                return _GenerateLumaColourVariationsFromDCSImages;
            }
            set
            {
                _GenerateLumaColourVariationsFromDCSImages = value;
            }
        }

        public bool GenerateLumaColourVariationsFromCustomThreeChannelImages
        {
            get
            {
                return _GenerateLumaColourVariationsFromCustomThreeChannelImages;
            }
            set
            {
                _GenerateLumaColourVariationsFromCustomThreeChannelImages = value;
            }
        }

        public bool GenerateLumaColourVariationsFromTintedImages
        {
            get
            {
                return _GenerateLumaColourVariationsFromTintedImages;
            }
            set
            {
                _GenerateLumaColourVariationsFromTintedImages = value;
            }
        }

        public bool GenerateLumaColourVariationsFromGradientMappedImages
        {
            get
            {
                return _GenerateLumaColourVariationsFromGradientMappedImages;
            }
            set
            {
                _GenerateLumaColourVariationsFromGradientMappedImages = value;
            }
        }

        /// <summary>
        /// Controls whether or not DCS variations will be created 
        /// based on three-channel images whose channels are not combined variations
        /// For example, NIR-R-G.
        /// Example: 
        /// </summary>
        public bool GenerateDCSImagesFromIndividualChannels
        {
            get
            {
                return _GenerateDCSImagesFromIndividualChannels;
            }
            set
            {
                _GenerateDCSImagesFromIndividualChannels = value;
            }
        }

        /// <summary>
        /// Controls whether or not DCS variations will be created 
        /// based on three-channel images whose channels are combined variations
        /// For example,(NIR_R(Avg))-G-B
        /// Example: 
        /// </summary>
        public bool GenerateDCSImagesFromCombinedChannels
        {
            get
            {
                return _GenerateDCSImagesFromCombinedChannels;
            }
            set
            {
                _GenerateDCSImagesFromCombinedChannels = value;
            }
        }

        /// <summary>
        /// Controls whether or not DCS variations will be created 
        /// based on the overall min/max/average image
        /// Example: 
        /// </summary>
        public bool GenerateDCSImagesFromCustomThreeChannelImages
        {
            get
            {
                return _GenerateDCSImagesFromCustomThreeChannelImages;
            }
            set
            {
                _GenerateDCSImagesFromCustomThreeChannelImages = value;
            }
        }

        /// <summary>
        /// Controls whether or not DCS variations will be used to
        /// create DCS-enhanced-contrast images as well.
        /// Example: 
        /// </summary>
        public bool GenerateDCSEnhancedContrast
        {
            get
            {
                return _GenerateDCSEnhancedContrast;
            }
            set
            {
                _GenerateDCSEnhancedContrast = value;
            }
        }

        public DCSParameterSet[] DCSTypes
        {
            get
            {
                return _DCSTypes;
            }
            set
            {
                _DCSTypes = value;
            }
        }

        #endregion

        public TMSBProcessingConfiguration()
        {
            ScriptVariables = new ScriptVariableCollection();

            //ColourImageOverallOutputNormalization = NormalizationMode.None;
            ColourImageOutputTransformations = new TransformationProfileCollection();

            ChannelCombinationMethods = new CombinedBandConfig[0];
            _StatisticalGreyscaleCubeConfigurations = new StatisticalCubeConfig[0];

            CalculatedChannelConfigurations = new CalculatedChannelCombinationConfig[0];

            CustomGreyscaleChannelConfigurations = new CustomGreyscaleChannelConfig[0];

            CustomThreeChannelImageConfigurations = new CustomThreeChannelImageConfig[0];

            _TintMappings = new TintMapping[0];

            _GradientMappings = new TintMapping[0];

            GenerateStatisticalImageThreeChannelPermutations = false;
            GenerateStatisticalImageThreeChannelPermutationsUnrestricted = false;

            GenerateStatisticalChannels = true;
            GenerateCalculatedChannels = true;
            GenerateCustomChannels = true;
            GenerateTintedImages = true;
            GenerateGradientImages = true;
            GenerateThreeChannelImages = true;
            GenerateStandardThreeChannelImages = true;
            GenerateCustomThreeChannelImages = true;
            GenerateCalculatedThreeChannelImages = true;
            GenerateLumaColourImages = true;
            GenerateDCSImages = true;

            _GenerateLumaColourVariationsFromIndividualBands = false;
            _GenerateLumaColourVariationsFromCombinedChannels = false;
            _GenerateLumaColourVariationsFromStatisticalChannels = false;
            _GenerateLumaColourVariationsFromThreeChannelImages = false;
            _GenerateLumaColourVariationsFromDCSImages = false;
            _GenerateLumaColourVariationsFromCustomThreeChannelImages = false;
            _GenerateLumaColourVariationsFromTintedImages = false;
            _GenerateLumaColourVariationsFromGradientMappedImages = false;

            _GenerateDCSImagesFromIndividualChannels = false;
            _GenerateDCSImagesFromCombinedChannels = false;
            _GenerateDCSImagesFromCustomThreeChannelImages = false;

            _DCSTypes = new DCSParameterSet[0];
        }

        public void AddCustomGreyscaleChannelConfig(CustomGreyscaleChannelConfig newConfig)
        {
            if ((CustomGreyscaleChannelConfigurations == null) || (CustomGreyscaleChannelConfigurations.Length == 0))
            {
                CustomGreyscaleChannelConfigurations = new CustomGreyscaleChannelConfig[1];
            }
            else
            {
                Array.Resize(ref _CustomGreyscaleChannelConfigurations, _CustomGreyscaleChannelConfigurations.Length + 1);
            }
            CustomGreyscaleChannelConfigurations[CustomGreyscaleChannelConfigurations.Length - 1] = newConfig;
        }

        public void AddChannelCombinationMethod(CombinedBandConfig newConfig)
        {
            if (CombinedBandConfig.ArrayContainsBandCombineMethod(ChannelCombinationMethods, newConfig.BandCombineMode))
            {
                throw new ArgumentException("An attempt was made to add an additional combined channel greyscale variation based on the '" + newConfig.BandCombineMode.ToString() + "' method. Multiple variations using the same combination method are not currently supported.");
            }
            Array.Resize(ref _ChannelCombinationMethods, _ChannelCombinationMethods.Length + 1);
            ChannelCombinationMethods[ChannelCombinationMethods.Length - 1] = newConfig;
        }

        public void AddStatisticalCubeConfiguration(StatisticalCubeConfig newConfig)
        {
            foreach (StatisticalCubeConfig currentConfig in StatisticalGreyscaleCubeConfigurations)
            {
                if (currentConfig.CubeName.ToLower().Trim() == newConfig.CubeName.ToLower().Trim())
                {
                    throw new ArgumentException("An attempt was made to add multiple greyscale statistical cube configurations with the same cube name ('" +
                        currentConfig.CubeName + "'). Names must be unique. Please rename one of the cubes in this configuration file.");
                }
            }
            Array.Resize(ref _StatisticalGreyscaleCubeConfigurations, _StatisticalGreyscaleCubeConfigurations.Length + 1);
            _StatisticalGreyscaleCubeConfigurations[_StatisticalGreyscaleCubeConfigurations.Length - 1] = newConfig;
        }

        public void AddCalculatedChannelConfiguration(CalculatedChannelCombinationConfig newConfig)
        {
            if ((CalculatedChannelConfigurations == null) || (CalculatedChannelConfigurations.Length == 0))
            {
                CalculatedChannelConfigurations = new CalculatedChannelCombinationConfig[1];
            }
            else
            {
                Array.Resize(ref _CalculatedChannelConfigurations, _CalculatedChannelConfigurations.Length + 1);
            }
            CalculatedChannelConfigurations[CalculatedChannelConfigurations.Length - 1] = newConfig;
        }

        //public void AddStatisticalFalseColourConfiguration(StatisticalFalseColourImageConfig newConfig)
        //{
        //    Array.Resize(ref _StatisticalFalseColourConfigurations, _StatisticalFalseColourConfigurations.Length + 1);
        //    StatisticalFalseColourConfigurations[StatisticalFalseColourConfigurations.Length - 1] = newConfig;
        //}

        public void AddCustomThreeChannelImageConfig(CustomThreeChannelImageConfig newConfig)
        {
            if ((CustomThreeChannelImageConfigurations == null) || (CustomThreeChannelImageConfigurations.Length == 0))
            {
                CustomThreeChannelImageConfigurations = new CustomThreeChannelImageConfig[1];
            }
            else
            {
                Array.Resize(ref _CustomThreeChannelImageConfigurations, _CustomThreeChannelImageConfigurations.Length + 1);
            }
            CustomThreeChannelImageConfigurations[CustomThreeChannelImageConfigurations.Length - 1] = newConfig;
        }

        public void AddDCSType(DCSMode mode, DCSColourSpace colourSpace, float rdcsPercent)
        {
            DCSParameterSet newType = new DCSParameterSet();
            newType.Mode = mode;
            newType.ColourSpace = colourSpace;
            newType.RDCSSectionPercent = rdcsPercent;

            AddDCSType(newType);
        }

        public void AddDCSType(DCSParameterSet newDCSType)
        {
            Array.Resize(ref _DCSTypes, _DCSTypes.Length + 1);
            _DCSTypes[_DCSTypes.Length - 1] = newDCSType;
        }

        public void AddTintMapping(TintMapping newTintMapping)
        {
            Array.Resize(ref _TintMappings, _TintMappings.Length + 1);
            _TintMappings[_TintMappings.Length - 1] = newTintMapping;
        }

        public void AddGradientMapping(TintMapping newGradientMapping)
        {
            Array.Resize(ref _GradientMappings, _GradientMappings.Length + 1);
            GradientMappings[GradientMappings.Length - 1] = newGradientMapping;
        }

    }
}
