// BenLincoln.DaVinci - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.DaVinci.

// BenLincoln.DaVinci is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.DaVinci is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.DaVinci (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace BenLincoln.DaVinci
{
    public struct TintingColourXMLParameters
    {
        public string TintingColourName;
        public ColourXMLParameters ColourParameters;
    }

    public class TintingColour : IComparable
    {
        protected string _TintingColourName;
        protected string _BasePlaneName;

        protected Colour _Colour;

        protected const string _TemporaryObjectNamePrefix = "tint_temp_";

        #region Properties

        public string TintingColourName
        {
            get
            {
                return _TintingColourName;
            }
            set
            {
                _TintingColourName = value;
            }
        }

        public string BasePlaneName
        {
            get
            {
                return _BasePlaneName;
            }
            set
            {
                _BasePlaneName = value;
            }
        }

        public Colour Colour
        {
            get
            {
                return _Colour;
            }
            set
            {
                _Colour = value;
            }
        }

        #endregion

        public TintingColour()
        {
            _TintingColourName = "";
            _BasePlaneName = "";
            //_Red = 0.0f;
            //_Green = 0.0f;
            //_Blue = 0.0f;
            _Colour = new Colour();
        }

        public TintingColour(string tintingColourName, string basePlaneName, float red, float green, float blue)
            : this()
        {
            TintingColourName = tintingColourName;
            BasePlaneName = basePlaneName;
            //if ((red < 0.0f) || (red > 1.0f) || (green < 0.0f) || (green > 1.0f) || (blue < 0.0f) || (blue > 1.0f))
            //{
            //    throw new ArgumentOutOfRangeException("The red, green, and blue values must be greater than zero but less than one.");
            //}
            Colour.Red = red;
            Colour.Green = green;
            Colour.Blue = blue;
            //IncludeInSecondOrderImages = true;
        }

        //public TintingColour()
        //    : this("", "", 0.0f, 0.0f, 0.0f)
        //{
        //}

        public static TintingColour FromXMLParameters(TintingColourXMLParameters xmlParams)
        {
            try
            {
                Colour.RGBValueFormat vFormat = Colour.RGBValueFormatFromString(xmlParams.ColourParameters.Format);
                switch (vFormat)
                {
                    case Colour.RGBValueFormat.Byte8:
                        TintingColour tc = From8BitRGB(xmlParams.TintingColourName, "", (int)xmlParams.ColourParameters.Red, (int)xmlParams.ColourParameters.Green,
                            (int)xmlParams.ColourParameters.Blue);
                        //tc.IncludeInSecondOrderImages = xmlParams.IncludeInSecondOrderImages;
                        return tc;
                        break;
                    case Colour.RGBValueFormat.UInt16:
                        TintingColour tc2 = From16BitRGB(xmlParams.TintingColourName, "", (int)xmlParams.ColourParameters.Red, (int)xmlParams.ColourParameters.Green,
                            (int)xmlParams.ColourParameters.Blue);
                        //tc2.IncludeInSecondOrderImages = xmlParams.IncludeInSecondOrderImages;
                        return tc2;
                        break;
                    case Colour.RGBValueFormat.Float32:
                        TintingColour tc3 = new TintingColour(xmlParams.TintingColourName, "", xmlParams.ColourParameters.Red, xmlParams.ColourParameters.Green,
                            xmlParams.ColourParameters.Blue);
                        //tc3.IncludeInSecondOrderImages = xmlParams.IncludeInSecondOrderImages;
                        return tc3;
                        break;
                }
            }
            catch (ArgumentException argEx)
            {
                throw new ArgumentException("Could not create a tinting colour from the passed XML parameters (" +
                    "colour name: '" + xmlParams.TintingColourName + "'). Verify that the Format value is valid.", argEx);
            }
            catch (Exception ex)
            {
                throw new ArgumentException("Could not create a tinting colour from the passed XML parameters (" +
                    "colour name: '" + xmlParams.TintingColourName + "').", ex);
            }
            throw new ArgumentException("An unexpected result occurred attempting to create the tinting colour named '" +
                xmlParams.TintingColourName + "'. Please verify that the XML definition conforms to the required specifications.");
        }

        public static TintingColour From16BitRGB(string tintingColourName, string basePlaneName, int red, int green, int blue)
        {
            TintingColour tC = new TintingColour();
            tC.TintingColourName = tintingColourName;
            tC.BasePlaneName = basePlaneName;
            tC.Colour = Colour.From16BitRGB(red, green, blue);
            return tC;
        }

        public static TintingColour From8BitRGB(string tintingColourName, string basePlaneName, int red, int green, int blue)
        {
            TintingColour tC = new TintingColour();
            tC.TintingColourName = tintingColourName;
            tC.BasePlaneName = basePlaneName;
            tC.Colour = Colour.From8BitRGB(red, green, blue);
            return tC;
        }

        public string GetCreationScriptCode(string scriptTargetObjectName, string scriptTemplateObjectName)
        {
            //string scriptTargetObjectName = _TintingColourName;
            StringBuilder builder = new StringBuilder();
            builder.Append("# Create the '");
            builder.Append(_TintingColourName);
            builder.Append("' tinting colour image");
            builder.Append(Environment.NewLine);

            // create an image of the correct dimensions automatically by starting with an existing image
            // and cloning it twice
            builder.Append(scriptTargetObjectName);
            builder.Append(" = float(");
            //builder.Append(_BasePlaneName);
            builder.Append(scriptTemplateObjectName);
            builder.Append(")");
            builder.Append(Environment.NewLine);

            builder.Append(scriptTargetObjectName);
            builder.Append(" = threeclone(");
            builder.Append(scriptTargetObjectName);
            builder.Append(")");
            builder.Append(Environment.NewLine);

            // set the RGB values by plane
            builder.Append(scriptTargetObjectName);
            builder.Append("[0,0,1] = ");
            builder.Append(BenLincoln.Utility.FormatConverter.FloatToString(Colour.Red));
            builder.Append(Environment.NewLine);

            builder.Append(scriptTargetObjectName);
            builder.Append("[0,0,2] = ");
            builder.Append(BenLincoln.Utility.FormatConverter.FloatToString(Colour.Green));
            builder.Append(Environment.NewLine);

            builder.Append(scriptTargetObjectName);
            builder.Append("[0,0,3] = ");
            builder.Append(BenLincoln.Utility.FormatConverter.FloatToString(Colour.Blue));
            builder.Append(Environment.NewLine);

            return builder.ToString();
        }

        public int CompareTo(object obj)
        {
            if (obj is TintingColour)
            {
                TintingColour b2 = (TintingColour)obj;
                return _TintingColourName.CompareTo(b2.TintingColourName);
            }
            else
            {
                return _TintingColourName.CompareTo(obj.ToString());
            }
        }


    }
}
