// BenLincoln.Utility - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.Utility.

// BenLincoln.Utility is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.Utility is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.Utility (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace BenLincoln.Utility
{
    public class DirectoryLister
    {
        protected int _MaximumDirectoryRecursion;
        protected ArrayList _Errors;

        #region Properties

        public int MaximumDirectoryRecursion
        {
            get
            {
                return _MaximumDirectoryRecursion;
            }
            set
            {
                _MaximumDirectoryRecursion = value;
            }
        }

        #endregion

        public DirectoryLister()
        {
            _MaximumDirectoryRecursion = 128;
            _Errors = new ArrayList();
        }

        public string[] GetRecursiveDirectoryList(string baseDirectory, out string[] errorMessages)
        {
            return GetRecursiveDirectoryList(new string[] { baseDirectory }, out errorMessages);
        }

        public string[] GetRecursiveDirectoryList(string[] baseDirectories, out string[] errorMessages)
        {
            _Errors = new ArrayList();
            string[] result = GetRecursiveDirectoryList(baseDirectories, 1);
            if (_Errors.Count > 0)
            {
                errorMessages = (string[])_Errors.ToArray("".GetType());
            }
            else
            {
                errorMessages = new string[0];
            }
            return result;
        }

        protected string[] GetRecursiveDirectoryList(string baseDirectory, int currentRecursionLevel)
        {
            return GetRecursiveDirectoryList(new string[] { baseDirectory }, currentRecursionLevel);
        }

        protected string[] GetRecursiveDirectoryList(string[] baseDirectories, int currentRecursionLevel)
        {
            ArrayList result = new ArrayList();

            foreach (string baseDir in baseDirectories)
            {
                if (!result.Contains(baseDir))
                {
                    // only add directories to the list if they actually contain at least one file
                    if (Directory.GetFiles(baseDir).Length > 0)
                    {
                        result.Add(baseDir);
                    }

                    if (currentRecursionLevel < _MaximumDirectoryRecursion)
                    {
                        try
                        {
                            string[] subDirPaths = Directory.GetDirectories(baseDir);

                            foreach (string subDirPath in subDirPaths)
                            {
                                try
                                {
                                    string[] subDirs = GetRecursiveDirectoryList(subDirPath, currentRecursionLevel + 1);
                                    foreach (string subDir in subDirs)
                                    {
                                        result.Add(subDir);
                                    }
                                }
                                catch (Exception ex)
                                {
                                    _Errors.Add("Unable to access the directory '" + baseDir + "'. " +
                                        "The error message was '" + ex.Message + "'.");
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            _Errors.Add("Unable to scan the directory '" + baseDir + "' for subdirectories. " +
                                "The error message was '" + ex.Message + "'.");
                        }
                    }
                    else
                    {
                        _Errors.Add("The maximum depth of directory recursion was reached with the directory '" + baseDir +
                            "'. Directories below this level will not be scanned.");
                    }
                }
            }

            return (string[])result.ToArray("".GetType());
        }
    }
}
