// BenLincoln.Utility - A shared library used by The Mirror's Surface Breaks
// Copyright 2010-2011 Ben Lincoln
// http://www.beneaththewaves.net/
//

// This file is part of BenLincoln.Utility.

// BenLincoln.Utility is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.Utility is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.Utility (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace BenLincoln.Utility
{
    public class GenericObjectComparer : IComparer
    {
        protected PropertyDescriptor _SortProperty;
        protected ListSortDirection _SortDirection;

        public GenericObjectComparer(PropertyDescriptor sortProperty,
                        ListSortDirection sortDirection)
        {
            _SortProperty = sortProperty;
            _SortDirection = sortDirection;
        }

        public int Compare(object x, object y)
        {
            int result = 0;

            if (_SortProperty.PropertyType.IsEnum)
            {
                result = (int)_SortProperty.GetValue(x) - (int)_SortProperty.GetValue(y);
            }
            else
            {
                if (_SortProperty.PropertyType == typeof(int))
                {
                    result = (int)_SortProperty.GetValue(x) - (int)_SortProperty.GetValue(y);
                }
                else if (_SortProperty.PropertyType == typeof(string))
                {
                    result = string.Compare((string)_SortProperty.GetValue(x), (string)_SortProperty.GetValue(y));
                }
                else if (_SortProperty.PropertyType == typeof(bool))
                {
                    int boolX = 0;
                    int boolY = 0;
                    if (((bool)_SortProperty.GetValue(x)) == true)
                    {
                        boolX = 1;
                    }
                    if (((bool)_SortProperty.GetValue(y)) == true)
                    {
                        boolY = 1;
                    }
                    result = boolX - boolY;
                }
                else if (_SortProperty.PropertyType == typeof(DateTime))
                {
                    result = DateTime.Compare((DateTime)_SortProperty.GetValue(x), (DateTime)_SortProperty.GetValue(y));
                }
                else
                {
                    result = string.Compare(_SortProperty.GetValue(x).ToString(), _SortProperty.GetValue(y).ToString());
                }
            }

            if (_SortDirection == ListSortDirection.Descending)
            {
                result = -result;
            }

            return result;
        }
    }
}
